@@ -48,7 +48,7 @@ oldVerbosity = Screen('Preference', 'Verbosity', 1);
 oldSyncTests = Screen('Preference', 'SkipSyncTests', 2);
 
 try
-  win = Screen('OpenWindow', 0, 0, [0 0 32 32]);
+  win = Screen('OpenWindow', 0, 0, [0 0 32 32], 24);
   winfo = Screen('GetWindowInfo', win);
   modesettingddxactive = Screen('GetWindowInfo', win, 8);
   Screen('CloseAll');
@@ -184,7 +184,7 @@ try
         if ismember(i, remainingOutputs)
           scanout = outputs{i};
           winRect = OffsetRect([0, 0, 600, 100], scanout.xStart, scanout.yStart);
-          w(i) = Screen('Openwindow', scanout.screenNumber, 0, winRect);
+          w(i) = Screen('Openwindow', scanout.screenNumber, 0, winRect, 24);
           Screen('TextSize', w(i), 48);
           Screen('DrawText', w(i), [num2str(i) ') ' scanout.name], 10, 10, 255);
           Screen('Flip', w(i));
@@ -294,6 +294,7 @@ try
       fprintf('outputs. It also causes substantial reduction in graphics performance, and only works with\n');
       fprintf('some desktop GUI environments, e.g., GNOME-3. This function is highly experimental and may\n');
       fprintf('not work at all on your setup, so use a photometer to verify actual precision carefully!\n');
+      fprintf('This also works with later gpus up to and including AMD Vega, but does not need setup here.\n');
       fprintf('Answer no here if a 10 bpc / 30 bpp framebuffer is sufficient for your needs.\n');
       atinotiling = '';
       while isempty(atinotiling) || ~ismember(atinotiling, ['y', 'n', 'd'])
@@ -311,7 +312,7 @@ try
         strcmp(xdriver, 'ati') || ((xversion(1) > 1 || (xversion(1) == 1 && xversion(2) >= 20)) && (strcmp(xdriver, 'modesetting') || strcmp(xdriver, 'amdgpu'))))
       fprintf('\n\nDo you want to setup a 30 bit framebuffer for 10 bpc precision per color channel?\n');
       if strcmp(xdriver, 'intel') || strcmp(xdriver, 'nouveau') || strcmp(xdriver, 'ati') || strcmp(xdriver, 'modesetting') || strcmp(xdriver, 'amdgpu')
-        fprintf('This will need the very latest drivers, so a year 2018 or later Linux distribution, e.g., at least Ubuntu 18.04 LTS,\n');
+        fprintf('This will need a year 2018 or later Linux distribution, e.g., at least Ubuntu 18.04 LTS,\n');
         fprintf('with Mesa 18.0 or later for Intel and AMD gpus, and Mesa 18.1 or later for NVidia gpus.\n');
       end
 
@@ -320,9 +321,11 @@ try
       end
 
       fprintf('If your desktop GUI fails to work, or Psychtoolbox gives lots of timing or page-flip related warnings,\n');
-      fprintf('then you know your system and hardware is not ready yet for this depth 30 mode. On AMD hardware from 2007 or later\n');
-      fprintf('depth 30 will always work even without the need to set it up here, at least for PsychImaging native 10 bit\n');
-      fprintf('framebuffer tasks, albeit at potentially slightly lower performance. AMD gpus have special support in PTB in this sense.\n');
+      fprintf('then you know your system and hardware is not ready yet for this depth 30 mode. On AMD hardware sold\n');
+      fprintf('from 2007 to ~2019, up to and including AMD Vega, but *not* anymore for AMD Navi RX 5000 or AMD Ryzen\n');
+      fprintf('processor integrated graphics or later models, depth 30 will always work even without the need to set\n');
+      fprintf('it up here, at least for PsychImaging native 10 bit framebuffer tasks, albeit at potentially slightly\n');
+      fprintf('lower performance. These slightly older AMD gpus have special support by PTB in this sense.\n');
       fprintf('Also note that not all gpus can output true 10 bpc on all types of video outputs. Check carefully with a photometer etc.!\n');
 
       depth30bpp = '';
@@ -405,6 +408,14 @@ try
           xdriver = 'modesetting';
           modesetting = 'y';
         end
+
+        % If the user explicitly does not want modesetting, and there are no forcing circumstances to use or not use it,
+        % (aka modesetting == 'd') and we are actually running on a Intel gpu or nouveau/NVidia, then force modesetting
+        % off. Background: As of Ubuntu 20.04-LTS, the distro will select modesetting-ddx by default for both Intel and
+        % NVidia+nouveau, so we need a config file to opt-out of use of modesetting-ddx, not to opt-in.
+        if (usemodesetting == 'n') && (modesetting == 'd') && (strcmp(xdriver, 'intel') || strcmp(xdriver, 'nouveau'))
+          modesetting = 'n';
+        end
       end
     end
 
@@ -813,14 +824,14 @@ function xdriver = DetectDDX(winfo)
       % Controlled by the open-source drivers. GPU minor
       % type defines the DCE generation and that in turn
       % usually predicts the x driver well:
-      if winfo.GPUMinorType >= 100
-        % DCE-10 or later, ergo "Volcanic Islands" family or
-        % later -> amdgpu ddx:
+      if winfo.GPUMinorType >= 100 || winfo.GPUMinorType == -1
+        % DCE-10 or later, ergo "Volcanic Islands" family or later. Or a brand-new
+        % DCN class gpu (Ryzen+ APU's, Navi+, ...) or later -> amdgpu ddx:
         fprintf('Recent AMD GPU with open-source driver detected. ');
         xdriver = 'amdgpu';
       else
-        % DCE-8 or earlier, or too new to be supported (DCN-x) -> ati ddx:
-        fprintf('Classic or unknown AMD GPU with open-source driver detected. ');
+        % DCE-8 or earlier => ati ddx:
+        fprintf('Classic AMD GPU with open-source driver detected. ');
         xdriver = 'ati';
       end
     else
