@@ -1,5 +1,5 @@
-function FlipTimingWithRTBoxPhotoDiodeTest(configFile, targetFolder)
-% FlipTimingWithRTBoxPhotoDiodeTest([configFile][, targetFolder])
+function FlipTimingWithRTBoxPhotoDiodeTest(configFile, targetFolder, usevulkan, bpc)
+% FlipTimingWithRTBoxPhotoDiodeTest([configFile][, targetFolder][, usevulkan=0][, bpc=8])
 %
 % Test visual stimulus onset timing accuracy and visual stimulus onset
 % timestamping precision and robustness under varying loads, conditions and
@@ -22,6 +22,15 @@ function FlipTimingWithRTBoxPhotoDiodeTest(configFile, targetFolder)
 % selected. If a folder is specified, that folder is used if it is writable, otherwise
 % the users home directory is tried as target.
 %
+% 'usevulkan' If 1, try to use a Vulkan display backend instead of the
+% OpenGL display backend. See 'help PsychVulkan'.
+%
+% 'bpc' Request a specific output framebuffer color precision. Currently
+% supported are 8 for standard 8 bpc RGBA8 framebuffer, 10 bpc for RGB10A2,
+% and 16 bpc for a RGBA16F floating point framebuffer. Defaults to 8 bpc,
+% which is the only precision that is guaranteed to be supported on all
+% operating systems, graphics cards and displays.
+%
 % Mandatory variables in the config file, part of the struct variable 'conf':
 % ---------------------------------------------------------------------------
 %
@@ -71,6 +80,15 @@ if (nargin >= 2) && ~isempty(targetFolder)
     basepath = targetFolder;
 end
 
+% Default to standard OpenGL backend instead of Vulkan by default:
+if nargin < 3 || isempty(usevulkan)
+    usevulkan = 0;
+end
+
+if nargin < 4 || isempty(bpc)
+    bpc = 8;
+end
+
 % Is it the standard path for config files?
 if strcmp([PsychtoolboxRoot 'PsychTests' filesep 'TestConfigurations'], basepath)
     % Yes. Then assume target folder is our standard folder in PsychTests.
@@ -244,16 +262,40 @@ if useRTbox == 2
 end
 
 try
+    PsychImaging('PrepareConfiguration');
+
+    if usevulkan
+        % Use PsychVulkan display backend instead of standard OpenGL:
+        PsychImaging('AddTask', 'General', 'UseVulkanDisplay');
+    end
+
+    switch bpc
+        case 8
+            % Nothing to do.
+        case 10
+            PsychImaging('AddTask', 'General', 'EnableNative10BitFramebuffer');
+        case 16
+            PsychImaging('AddTask', 'General', 'EnableNative16BitFloatingPointFramebuffer');
+        otherwise
+            error('Invalid bpc specified!');
+    end
+
+    conf.bpc = bpc;
+
     if useRTbox == -1
         % Setup Datapixx mode for timestamping:
-        PsychImaging('PrepareConfiguration');
         PsychImaging('AddTask', 'General', 'UseDataPixx');
-        [w, winrect] = PsychImaging('OpenWindow', res.screenId, 0, [], [], [], conf.Stereo);
-    else
-        [w, winrect] = Screen('OpenWindow', res.screenId, 0, [], [], [], conf.Stereo);
     end
+
+    [w, winrect] = PsychImaging('OpenWindow', res.screenId, 0, [], [], [], conf.Stereo);
     res.winfo = Screen('GetWindowInfo', w);
 
+    % Needed for Vulkan testing on drivers without Vulkan interop, where only
+    % alternation between black and white frames happens, out of our control.
+    % So for the photodiode or Videoswitcher methods to work at all, we need
+    % to start off with the right frame, and this flip achieves that:
+    Screen('Flip', w);
+
     % Switch to selectable realtime-priority level to reduce timing jitter
     % and interruptions caused by other applications and the operating
     % system itself:
@@ -657,7 +699,7 @@ try
     save(res.outFilename, 'res', '-V6');
     res.outFilename = [];
 
-    if ~IsOSX
+    if 1
         % Plot all our measurement results:
 
         % Figure 1 shows time deltas between successive flips in milliseconds:
@@ -717,7 +759,7 @@ try
     onsets = res.onsetFlipTime(1:i);
     % vbls: onsets = res.vblFlipTime(1:i);
     difference = (onsets - res.measuredTime) * 1000;
-    if ~IsOSX
+    if 1
         plot(difference);
         title('Difference dt = FlipOnset - MeasurementBOX in msecs:');
     end
